/***************************************************************************
    NWNXVaultster.cpp - Implementation of the CNWNXVaultster class for Linux.
    Copyright (C) 2003 Jeroen Broekhuizen (nwnx@jengine.nl)
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ***************************************************************************
    05-07-2005:
       - Added check for trailing backslash in servervault
 
 ***************************************************************************/
 
#include "NWNXVaultster.h"

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <pthread.h>
#include <stddef.h>

#include "sock.h"

CNWNXVaultster::CNWNXVaultster()
{
	confKey = "Vaultster";
} 

CNWNXVaultster::~CNWNXVaultster()
{
}

bool CNWNXVaultster::OnCreate (gline *config, const char* LogDir)
{
	pthread_t id;
	char log[128];
	bool validate = true, startServer = true;
	
	// call the base class function
	sprintf (log, "%s/nwnx_vaultster.txt", LogDir);
	if (!CNWNXBase::OnCreate(config,log))
		return false;
	
	// write copy information to the log file
	Log (0, "NWNX2 Vaultster version 0.3.0 Linux.\n");
	Log (0, "Copyright 2004 (C) Jeroen Broekhuizen\n");
	
	if(nwnxConfig->exists(confKey)) {
		// set up the clients
		maxClients = atoi ((*nwnxConfig)[confKey]["maxclients"].c_str());
		if (nwnxConfig->exists (confKey, "validateClient"))
			validate = atoi ((*nwnxConfig)[confKey]["validateclient"].c_str());
		if (nwnxConfig->exists (confKey, "startServer"))
			startServer = atoi ((*nwnxConfig)[confKey]["startServer"].c_str());
		
		// make sure that there is no trailing backslash
		string vault = (*nwnxConfig)[confKey]["vault"].c_str();
		if (vault[vault.length()-1] == '/')
			vault[vault.length()-1] = '\0';
		
		cClient::setServervault (vault.c_str());
		cClient::setCryptoKey ((*nwnxConfig)[confKey]["cryptokey"].c_str());
		cClient::setPassword ((*nwnxConfig)[confKey]["password"].c_str());
		cClient::setPort (atoi ((*nwnxConfig)[confKey]["port"].c_str()));
	
		server.setMaxClients (maxClients);
		server.setServervault (vault.c_str());
		server.setCryptoKey ((*nwnxConfig)[confKey]["cryptokey"].c_str());
		server.setPassword ((*nwnxConfig)[confKey]["password"].c_str());
		server.setPort (atoi ((*nwnxConfig)[confKey]["port"].c_str()));
		server.setValidate (validate);
		
		// load in the known servers
		if (validate) {
			int nr = atoi ((*nwnxConfig)["Link"]["count"].c_str());
			Log (0, "o Loading in %d known servers.\n", nr);
			for (int i = 1; i <= nr; i++) {
				char buf[256];
				sprintf (buf, "server%d", i);
				string name = (*nwnxConfig)["Link"][buf].c_str();
				Log (0, "o %s: %s\n", buf, name.c_str());
				server.addKnownServer (name);
			}
		}
		else {
			Log (0, "o Could not find known servers. No validation will take place.\n");
			server.setValidate (false);
		}
	}
	else {
		Log (0, "o Could not find configuration section!\n");
	}
	
	// initialize memory for the client structures
	clients = new cClient[maxClients];
	
	// start up the server
	if (startServer) {
		pthread_create (&id, 0, CServer::thread, &server);
		Log (0, "o Server started\n");
	}
	return true;
}

char* CNWNXVaultster::OnRequest(char* gameObject, char* Request, char* Parameters)
{
	int i;
	pthread_t id = 0;
	char* pos[2];
	
	// find out that should happen
	//Functions cmd = (Functions)atoi (Request);
	Functions cmd = (strcmp (Request,"GET")==0 ? Get : (strcmp (Request, "SEND")==0 ? Put : Status));
	Log (0, "o New request: %s\n", Request);
	if (cmd == Status) {
		// return status about current client
		int job = atoi (Parameters);
		Log (0, "Looking up status for client: %d\n", job);
		int status = clients[job].getStatus ();
		if (status == STATUS_ERROR)
			clients[job].setStatus (STATUS_OK);
		sprintf (Parameters, "%d", status);
		return NULL;
	}
	else if (cmd == Get || cmd == Put) {
		char* pos[2];
		DWORD id;
		int i;

		Log (0, "o New request: %d\n", (int)cmd);

		// find a not busy client spot in the array
		for (i = 0; i < maxClients; i++) {
			if (clients[i].getStatus() == STATUS_OK)
				break;
		}

		pos[0] = strchr (Parameters, '|');
		pos[1] = strchr (&pos[0][1], '|');
		
		if (i == maxClients || !pos[0] || !pos[1]) {
			Log (0, "o Too many clients already.\n");
			sprintf (Parameters, "-2");
			return NULL;
		}
		
		memset (clients[i].server, 0, 128);
		memset (clients[i].gamespy, 0, 128);
		memset (clients[i].character, 0, 32);
		strncpy (clients[i].server, Parameters, pos[0] - Parameters);
		strncpy (clients[i].gamespy, &pos[0][1], pos[1] - pos[0] - 1);
		strcpy  (clients[i].character, &pos[1][1]);
		clients[i].setCommand (cmd);

		Log (0, "o Handling client with id: %d\n", i);
		
		// start the client thread
		pthread_create (&id, NULL, cClient::thread, &clients[i]);
		sprintf (Parameters, "%d", i);
		return NULL;
	}
	else {
		// nothing changed in base class on Linux
		return 0;
	}
}

bool CNWNXVaultster::OnRelease()
{
	return true;
}

