#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <sys/stat.h>
#include <time.h>

#include "NWNXVaultster.h"
#include "client.h"
#include "findfile.h"

#include <iostream>
#include <cstdlib>
using namespace std;

typedef unsigned char uchar;
typedef unsigned long ulong;

// for logging purposes only
extern CNWNXVaultster vaultster;

cClient::cClient()
{
}

cClient::~cClient()
{
}

char cClient::serverVault[] = "";
char cClient::cryptoKey[] = "";
char cClient::password[] = "";
int cClient::port = 5100;

void strlwr (char* str)
{
	int len = strlen (str);
	for (int i = 0; i < len; i++)
		str[i] = tolower (str[i]);
}

void cClient::setCommand (Functions cmd)
{
   switch (cmd) {
   case Get:
      command = CMD_GET;
      break;
   case Put:
      command = CMD_SEND;
      break;
   }
}

void cClient::setStatus (int stat) 
{
	status = stat;
}

int cClient::getStatus ()
{
	return status;
}

void cClient::setPort (int p)
{
	port = p;
}

void cClient::setServervault (const char* vault)
{
	// save a static string with the servervault directory
	strcpy (serverVault, vault); 
}

void cClient::setCryptoKey (const char* key)
{
	// save a copy of the blowfish key
	strcpy (cryptoKey, key);
}

void cClient::setPassword (const char* pwd)
{
	// save the password for identification
	strcpy (password, pwd);
}

void* cClient::thread (void* param)
{
	cClient* client = (cClient*)param;
	
	client->setStatus (STATUS_BUSY);
	if (!client->run ()) {
		client->setStatus (STATUS_ERROR);
		vaultster.Log (0, "o An error occured during transmission.\n");
	}
	else {
		client->setStatus (STATUS_OK);
		vaultster.Log (0, "o Portalling was successfull,\n");
	}
	return 0;
}

bool cClient::run ()
{
	char filename[256], pattern[32];
	fish.Initialize ((unsigned char*)cryptoKey, strlen (cryptoKey));
	
	memset (filename, 0, 256);
	memset (pattern, 0, 32);
	
	// try to find the latest file
	createPattern (pattern);
	vaultster.Log (0, "o Start searching for %s\n", pattern);
	if (!findLatestFile (pattern, filename)) {
		vaultster.Log (0, "Could not find latest file for pattern %s\n", pattern);
		return false;
	}
	else
		vaultster.Log (0, "o Found file %s\n", filename);
		
	// try to connect to the server
	vaultster.Log (0, "o Trying to connect to server %s\n", server);
	socket.create ();
	if (!socket.connect (server, port)) {
		vaultster.Log (0, "Could not connect to server on port %d.\n", port);
		return false;
	}
	vaultster.Log (0, "o Connected\n");

	if (!handShake ()) {
		// wrong password, bail out!
		vaultster.Log (0, "Invalid password, or server rejected us.\n");
		socket.close ();
		return false;
	}
	else
		vaultster.Log (0, "o Using local file: %s\n", filename);
	
	if (!transmitFile (pattern, filename)) {
		// to bad! it failed
		vaultster.Log (0, "Transmitting file %s failed.\n", filename);
		socket.close ();
		return false;
	}
	
	socket.close ();
	return true;
}

void cClient::createPattern (char* pattern)
{
	// determine the length of the current pattern
	int characterLen = strlen (character);
	int patternLen = (characterLen >= 14) ? 14 : characterLen;
	
	// create the pattern string
	strncpy (pattern, character, characterLen);
	strlwr (pattern);
}

bool cClient::findLatestFile (char* pattern, char* filename)
{
	CFindFile find;
	char buffer[256], path[256];
	bool found = false;
	
	sprintf (path, "%s/%s", serverVault, gamespy);
	if (find.findFirstFile (path)) {
		char* name = NULL;
		int len =  strlen (pattern);
		struct stat stats;
		time_t last = 0;
		
		while (find.findNextFile ()) {
			// verify the pattern part to be correct
			if (strncmp (find.file, pattern, len) == 0) {
				sprintf (buffer, "%s/%s", path, find.file);
				stat (buffer, &stats);
				
				if (stats.st_mtime > last) {
					strcpy (filename, buffer);
					last = stats.st_mtime;
				}
			}
		}
		find.close ();
		
		// see if we actually found a file
		if (last != 0)
			found = true;
	}
	
	return found;
}

void cClient::generatePassword (char pwd[128])
{
	memset (pwd, 0, 128);
	strcpy (pwd, password);
	
	// build the dummy part of the password
	srand ((unsigned)time(NULL));
	for (int i = strlen(password); i < 128; i++)
		pwd[i] = 33 + (rand() % 90);
}

bool cClient::handShake ()
{
	int ret = 0;
	char pwd[128];
	uchar encrypted[128];
	
	// send the command to the server
	socket.send (command);
	socket.receive (ret);
	if (ret == INFO_NACK)
		return false;
	
	// now encrypt the password and send it over
	generatePassword (pwd);
	fish.Encode ((uchar*)pwd, encrypted, 128);
	socket.send (encrypted, 128);
	
	// hopefully the password is accepted
	socket.receive (ret);
	if (ret == INFO_NACK)
		return false;
	return true;
}

bool cClient::transmitFile (char* pattern, char* localFile)
{
	int ret = 0;
	sendEncrypted (gamespy);
	sendEncrypted (character);
	socket.receive (ret);
	if (ret == INFO_NACK)
		return false;

	// now perform the requested job
	switch (command) {
	case CMD_SEND:
		vaultster.Log (0, "o Sending file\n");
		return socket.sendFile (localFile);
	case CMD_GET:
		vaultster.Log (0, "o Receiving file\n");
		return socket.receiveFile (localFile);
	}
	
	// invalid command (should not get here)
	return false;
}

void cClient::sendEncrypted (char* str)
{
	uchar encrypted[256];
	
	// encode the path & send it to the server
	ulong patternLen = strlen (str);
	ulong len = fish.GetOutputLength (patternLen);
	fish.Encode ((uchar*)str, encrypted, patternLen);
	socket.send (len);
	socket.send (encrypted, len);
}

