#include <stdio.h>
#include <memory.h>
#include <pthread.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "NWNXVaultster.h"
#include "server.h"
#include "blowfish.h"
#include "findfile.h"

#include <iostream>
using namespace std;

// use for logging purposes only
extern CNWNXVaultster vaultster;


//////////////////////////////////////////////////////////////////////////////////////
// cServerClient class implementation

cServerClient::cServerClient()
{
	ready = true;
}

cServerClient::~cServerClient()
{
}

void* cServerClient::thread (void* param)
{
	cServerClient* client = (cServerClient*)param;
	if (client->run ())
		vaultster.Log (0, "o Job successfully finished.\n");
	else
		vaultster.Log (0, "o There was an error\n");
	client->setReady ();
	pthread_exit (0);
}

char cServerClient::serverVault[] = "";
char cServerClient::cryptoKey[] = "";
char cServerClient::password[] = "";

void cServerClient::setServervault (const char* vault)
{
        // save a static string with the servervault directory
        strcpy (serverVault, vault); 
}

void cServerClient::setCryptoKey (const char* key)
{
        // save a copy of the blowfish key
        strcpy (cryptoKey, key);
}

void cServerClient::setPassword (const char* pwd)
{
        // save the password for identification
        strcpy (password, pwd);
}

bool cServerClient::run ()
{
	char filename[256];
	
	// initialize the fish crypto engine
	fish.Initialize ((uchar*)cryptoKey, strlen (cryptoKey));
	
	vaultster.Log (0, "o Perform handshake\n");
	// try to perform the handshaking process
	if (!handShake ()) {
		vaultster.Log (0, "o Handshaking failed.\n");
		client.close ();
		return false;
	}
	vaultster.Log (0, "o Client is trusted. Retreiving file name...\n");
	
	// get the file name what this process is going about
	memset (filename, 0, 256);
	if (!getFileName (filename)) {
		vaultster.Log (0, "o Could not get filename to transmit.\n");
		client.close ();
		return false;
	}
	else
		vaultster.Log (0, "o Using file: %s\n", filename);
	
	// now process the command
	if (!processCommand (client, filename)) {
		vaultster.Log (0, "o Processing command failed.\n");
		return false;
	}
	
	client.close();
	return true;
}

bool cServerClient::handShake ()
{
	client.receive (cmd);
	if (cmd < CMD_SEND || cmd > CMD_QUERY) {
		client.send (INFO_NACK);
		return false;
	}
	else
		client.send (INFO_ACK);
		
	// do the handshaking
	if (cmd != CMD_QUERY) {
		int len = 218;
		uchar encrypted[128];
		char pwd[128];
		
		// see if the first part of thge decrypted password is correct
		client.receive (encrypted, len);
		fish.Decode (encrypted, (uchar*)pwd, 128);
		
		if (strncmp (pwd, password, strlen (password)) != 0) {
			client.send (INFO_NACK);
			return false;
		}
		else
			client.send (INFO_ACK);
	}
	return true;
}

void cServerClient::createPattern (char* pattern)
{
	// determine the length of the current pattern
	int characterLen = strlen (character);
	int patternLen = (characterLen >= 14) ? 14 : characterLen;
        
	// create the pattern string
	memset (pattern, 0, 32);
	strncpy (pattern, character, characterLen);
}

bool cServerClient::findLatestFile (char *path, char* pattern, char* filename)
{
	CFindFile find;
	char buffer[256];
	
	if (find.findFirstFile (path)) {
		char* name = NULL;
		int len =  strlen (pattern);
		struct stat stats;
		time_t last = 0;
		
		while (find.findNextFile ()) {
			// verify the pattern part to be correct
			if (strncmp (find.file, pattern, len) == 0) {
				sprintf (buffer, "%s/%s.bic", path, find.file);
				stat (buffer, &stats);
				
				if (stats.st_mtime > last) {
					strcpy (filename, buffer);
					last = stats.st_mtime;
				}
			}
		}
		find.close ();
		
		// see if we actually found a file
		if (last == 0)
			return false;
		return true;
	}
	else 
		return false;
}

bool cServerClient::getFileName (char* filename)
{
	int length;
	uchar encrypted[256];
	char pattern[32], path[260];
	
	memset (encrypted, 0, 256);
	memset (gamespy, 0, 128);
	memset (character, 0, 32);
	
	// receive the filename
	client.receive (length);
	client.receive (encrypted, length);
	fish.Decode (encrypted, (uchar*)gamespy, length);
	
	client.receive (length);
	client.receive (encrypted, length);
	fish.Decode (encrypted, (uchar*)character, length);

	createPattern (pattern);
	
	// try to find the specified file
	sprintf (path, "%s/%s", serverVault, gamespy);
	if (!findLatestFile (path, pattern, filename)) {
		if (cmd == INFO_SEND) {
			// set up a new name for the character
			sprintf (filename, "%s/%s/%s.bic", serverVault, gamespy, character);
			client.send (INFO_ACK);
		}
		else {
			// no filename found
			client.send (INFO_NACK);
			return false;
		}
	}
	else {
		// we found the file
		vaultster.Log (0, "o Could not find the latest file.\n"); 
		client.send (INFO_ACK);
	}
	return true;
}

bool cServerClient::processCommand (CSock& client, char* filename)
{
	switch (cmd) {
	case INFO_SEND:
		// receive the file
		return client.receiveFile(filename);
	case INFO_GET:
		// send the requested file to the client
		return client.sendFile(filename);
	}
	
	// invalid command
	return false;
}

/////////////////////////////////////////////////////////////////////////////////
// CServer class implementation

CServer::CServer()
{
	noValidate = true;
}

CServer::~CServer()
{
}

void CServer::setPort (int p)
{
   // save the port number, which is used to connect
   port = p;
}

void CServer::setMaxClients (int max)
{
   // keep in mind how much clients can portal at the same time
   maxClients = max;
}

void CServer::setServervault (const char* vault)
{
        // save a static string with the servervault directory
   cServerClient::setServervault (vault);
}

void CServer::setCryptoKey (const char* key)
{
        // save a copy of the blowfish key
   cServerClient::setCryptoKey (key);
}

void CServer::setPassword (const char* pwd)
{
        // save the password for identification
   cServerClient::setPassword (pwd);
}

void CServer::setValidate (bool validate)
{
	// must be negative for easy usage
	noValidate = !validate;
}

void CServer::addKnownServer (string server)
{
	knownServers.push_back (server);
}

void* CServer::thread (void* ptr)
{
	CServer* server = (CServer*)ptr;
	server->run ();
	pthread_exit (0);
}

bool CServer::run ()
{
	CSock client;
	pthread_t id;
	sockaddr_in addr;
	
	// try to open the port
	if (!socket.create (port))
		return false;
	socket.listen ();
	
	// initialize memory for the clients
	clients = new cServerClient[maxClients];
	
	// loop forever
	while (true) {
		if (socket.accept (client, addr)) {
			if (noValidate || isValidClient (addr)) {
				vaultster.Log (0, "o Serving client.\n");
				// find an empty spot to help this client
				int i = 0;
				for (i = 0; i < maxClients; i++) {
					if (clients[i].isReady()) {
						clients[i].setReady (false);
						clients[i].setSocket (client);
						pthread_create (&id, 0, cServerClient::thread, &clients[i]);
						break;
					}
				}
				// when no spot is found, close the connection
				if (i == maxClients) {
					client.close ();
				}
			}
			else {
				// invalid client, log was generated in isValidClient
				// send a nack by default on password checking
				int cmd = 0;
				client.receive (cmd);
				client.send (INFO_NACK);
				client.close ();
			}
		}
	}
	
	socket.close ();
}

bool CServer::isValidClient (sockaddr_in& client)
{
	// convert the ip to a string
	string addr = inet_ntoa (client.sin_addr);

	// try to find the client in our list
	vector<string>::iterator it = knownServers.begin();
	for (; it != knownServers.end(); it++) {
		if (isalpha ((*it)[0])) {
			// resolve IP address of the host
			hostent* host = gethostbyname ((*it).c_str());
			if (host) {
				struct in_addr *address = (struct in_addr *) host->h_addr;
				string server_addr = inet_ntoa(*address);
				if (addr == server_addr)
					return true;
			}
			else
				vaultster.Log (0, "* Warning(!!): could not retreive IP address for host.\n");
		}
		else {
			if ((*it) == addr)
				return true;
		}
	}

	// if we get here the client is not known, so reject
	vaultster.Log (0, "* Invalid client tries to connect: %s\n" , addr.c_str());
}
