#ifndef _SERVER_H_
#define _SERVER_H_

#include "sock.h"
#include "blowfish.h"
#include "../NWNXBase.h"
#include "constants.h"
#include <string>
#include <vector>
using namespace std;

#define INFO_ACK		0
#define INFO_NACK		1
#define INFO_MORE		2
#define INFO_READY		3

#define INFO_SEND		5
#define INFO_GET		6
#define INFO_QUERY		7
#define INFO_CLOSE		8

typedef unsigned char uchar;

class cServerClient
{
public:
	cServerClient();
	~cServerClient();
	
	static void* thread (void* param);
	
	static void setPassword (const char* pwd);
	static void setCryptoKey (const char* key);
	static void setServervault (const char* vault);
	
	inline void setSocket (CSock& s) { client = s; }
	inline void setReady (bool r = true) { ready = r; }
	inline bool isReady () { return ready; }
	
protected:
	bool run ();
	
	bool handShake ();
	void createPattern (char* pattern);
	bool findLatestFile (char* path, char* pattern, char* filename);
	bool getFileName (char* filename);
	bool processCommand (CSock& client, char* filename);
	
private:
	static char serverVault[256];
	static char cryptoKey[128];
	static char password[64];

	char gamespy[128];
	char character[32];

	CBlowFish fish;
	CSock client;
	bool ready;
	int cmd;
};

class CServer 
{
public:
	CServer();
	~CServer();
	
	static void* thread (void* ptr);
	
	void setPort (int port);
	void setMaxClients (int max);
	void setPassword (const char* pwd);
	void setCryptoKey (const char* key);
	void setServervault (const char* vault);
	void setValidate (bool validate);

	void addKnownServer (string server);

protected:
	bool run ();
	bool isValidClient (sockaddr_in& client);

private:
	CSock socket;

	vector<string> knownServers;
	
	int port;
	int maxClients;
	cServerClient* clients;
	bool noValidate;
};

#endif
