#include "sock.h"
#include <stdio.h>
#include <unistd.h>
#include <zlib.h>
#include <sys/stat.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <memory.h>
#include <dirent.h>
#include "NWNXVaultster.h"

#include <iostream>
using namespace std;

#define BUF_SIZE 1024
#define min(a,b) (((a)<(b))?(a):(b)) 

// for logging purposes only
extern CNWNXVaultster vaultster;

CSock::CSock()
{
	sock = -1;
	tv.tv_sec = 10;
	tv.tv_usec = 0;
}

CSock::CSock(const CSock& s): sock(s.sock)
{
	tv.tv_sec = 0;
	tv.tv_usec = 0;
}

CSock::~CSock()
{
}

bool CSock::create (int port)
{
	// open the socket
	sock = socket (AF_INET, SOCK_STREAM, 0);
	if (sock < 0)
		return false;
	
	// when a port is supplied, a server should be started
	if (port > 0) {
		sockaddr_in server;
		server.sin_family = AF_INET;
		server.sin_addr.s_addr = htonl (INADDR_ANY);
		server.sin_port = htons (port);
		
		// bind the server to the port
		if (bind (sock, (struct sockaddr*)&server, sizeof (server)) < 0)
			return false;
	}
	return true;
}

void CSock::close ()
{
	::close (sock);
	sock = -1;
}

void CSock::listen (int backlog)
{
	::listen (sock, backlog);
}

bool CSock::accept (CSock& client, sockaddr_in& addr)
{
	socklen_t len = sizeof (addr);
	client.sock = ::accept (sock, (sockaddr*)&addr, &len);
	if (client.sock < 0)
		return false;
	return true;
}

bool CSock::connect (char* serverName, int port)
{
	sockaddr_in server;
	server.sin_family = AF_INET;
	server.sin_port = htons (port);
	server.sin_addr.s_addr = inet_addr(serverName);
	
	// see if we have correct server information
	if (server.sin_addr.s_addr == INADDR_NONE) {
		hostent *host = gethostbyname (serverName);
		if (host == NULL)
			return false;
		memcpy (&server.sin_addr.s_addr, host->h_addr, host->h_length);
	}
	
	// now connect to the server
	if (::connect (sock, (sockaddr*)&server, sizeof(server)) == -1)
		return false;
	return true;
}

int CSock::send (const void* data, unsigned int size)
{
	fd_set fd;
	FD_ZERO(&fd);
	FD_SET(sock, &fd);
	
	tv.tv_sec = 5;
	tv.tv_usec = 0;
	
	// make sure we can write to the socket
	if (select (sock+1, NULL, &fd, NULL, &tv) > 0)
		return ::send (sock, (char*)data, size, 0);
	else
		return 0;
}

int CSock::receive (void* data, unsigned int size, int flags)
{
	fd_set fd;
	FD_ZERO(&fd);
	FD_SET(sock, &fd);
	
	tv.tv_sec = 5;
	tv.tv_usec = 0;
	
	// make sure that there is data waiting to be received
	if (select (sock+1, &fd, NULL, NULL, &tv) > 0)
		return recv (sock, (char*)data, size, flags);
	else
		return 0;
}

typedef unsigned char uchar;
typedef unsigned long ulong;

bool CSock::sendFile (char* filename, bool compressed)
{
	struct stat stats;
	uchar* filedata;
	int sended;
	vaultster.Log (0, "opening file.\n");
	
	// try to open the file
	FILE* file = fopen (filename, "rb");
	if (file == NULL)
		return false;
		
	vaultster.Log (0, "retreiving file stats\n");
	
	/*
	// get the file stats (size)
	if (stat (filename, &stats) != 0)
		return false;
	*/	
	vaultster.Log (0, "sending filesize to server\n");

	// send the info to the client
	//if (TIMEOUT (send (stats.st_size)))
	//	return false;
	//int realSize = stats.st_size;
	
	fseek (file, 0, SEEK_END);
	int realSize = ftell (file);
	if (realSize <= 0) {
		vaultster.Log (0, "o Could not retreive file size\n");
		return false;
	}
	else
		fseek (file, 0, SEEK_SET);
	
	if (TIMEOUT (send (realSize)))
		return false;
	
	vaultster.Log (0, "trying to compress data (%d bytes)\n", realSize);
	
	if (compressed) {
		// read in uncompressed data
		vaultster.Log (0, "reading in file data\n");
		uchar* uncompressedData = new uchar[realSize];
		fread (uncompressedData, 1, realSize, file);
		// compress in new buffer
		int compressedSize = int(realSize * 1.35f) + 12;
		vaultster.Log (0, "compressing the data\n");
		filedata = new uchar[compressedSize];
		int r = compress2 (filedata, (ulong*)&realSize, uncompressedData, (ulong)compressedSize, Z_DEFAULT_COMPRESSION);
		
		if (r == Z_BUF_ERROR) {
			delete[] filedata;
			compressedSize *= 2;
			filedata = new uchar[compressedSize];
			r = compress2 (filedata, (ulong*)&realSize, uncompressedData, (ulong)compressedSize, Z_DEFAULT_COMPRESSION);
		}
		
		vaultster.Log (0, "releasing memory\n");
		delete[] uncompressedData;
		// check if compression was successfull
		if (r != Z_OK) {
			switch (r) {
			case Z_BUF_ERROR:
				vaultster.Log (0, "o Destination buffer was to small.\n");
				break;
			case Z_MEM_ERROR:
				vaultster.Log (0, "o Insufficient memory.\n");
				break;
			}
			vaultster.Log (0, "o Compressing the data failed (error %d)!\n", r);
			return false;
		}
	}
	else {
		// no compression
		filedata = new uchar[realSize];
		fread (filedata, 1, realSize, file);
	}
	fclose (file);
	
	vaultster.Log (0, "o ready compressing\n");
	
	// now send the actual size of the (compressed-)file
	if (TIMEOUT (send (realSize)))
		return false;
		
	vaultster.Log (0, "o Transmission of file data (%d bytes).\n", realSize);
		
	int currentSend = 0;
	int bytesToSend = 0;
	char buf[BUF_SIZE];
	while (currentSend < realSize) {
		bytesToSend = min (BUF_SIZE, realSize-currentSend);
		memcpy (buf, &filedata[currentSend], bytesToSend);
		sended = send (buf, bytesToSend);
		if (sended == -1) {
			vaultster.Log (0, "o Send error, probably connection reset by host.\n");
			return false;
		}
		else if (TIMEOUT (sended)) {
			vaultster.Log (0, "o Time out\n");
			return false;
		}
		else
			vaultster.Log (0, "o Send %d bytes\n", sended);
		currentSend += sended;
	}
	delete[] filedata;
	
	int ret;
	receive (ret);
	if (ret != 1)
		return false;
	return true;
}

bool CSock::receiveFile (char* filename, bool compressed)
{
	int filesize, datasize, received;
	
	// receive end filesize
	if (TIMEOUT (receive (filesize)))
		return false;
	// receive actual bytes that gets transfered
	if (TIMEOUT (receive (datasize)))
		return false;
		
	// allocate memory for file content
	uchar* data = new uchar [datasize];
	if (data == NULL)
		return false;
		
	// receive the file contents
	uchar buf[BUF_SIZE];
	int currentRecv = 0, bytesToRecv = 0;
	while (currentRecv < datasize) {
		bytesToRecv = min (BUF_SIZE, datasize-currentRecv);
		received = receive (&data[currentRecv], bytesToRecv);
		if (TIMEOUT (received)) return false;
		currentRecv += received;
	}
	
	// now try to open the file
	FILE *file = fopen (filename, "wb");
	if (!file) {
		// try to create the directory
		char* pos = strrchr (filename, '/');
		*pos = 0x0; 
		
		DIR* d = opendir (filename);
		if (d == NULL)
			mkdir (filename, 0756);
		else {
			// there is something else wrong
			closedir (d);
			send (0);
			return false;
		}
		
		*pos = '/';
			
		// try to open it again
		file = fopen (filename, "wb");
		if (file == NULL) {
			send (0);
			return false;
		}
	}
	
	if (compressed) {
		// save uncompressed data in the file
		uchar* filedata = new uchar[filesize];
		uncompress (filedata, (ulong*)&filesize, data, (ulong)datasize);
		fwrite (filedata, 1, filesize, file);
		delete[] filedata;
	}
	else {
		// save normal data in the file
		fwrite (data, 1, filesize, file);
	}
	
	fclose (file);
	delete[] data;
	
	send (1);
	return true;
}
